#ifndef __CXXTEST__STDIOPRINTER_H
#define __CXXTEST__STDIOPRINTER_H

//
// The StdioPrinter is a simple TestListener that
// just prints "OK" if everything goes well, otherwise
// reports the error in the format of compiler messages.
// The StdioPrinter uses printf()
//

#include <cxxtest/ErrorFormatter.h>

#include <stdio.h>

namespace CxxTest 
{
    class StdioPrinter : public ErrorFormatter
    {
    public:
        StdioPrinter( FILE *o = stdout, const char *preLine = ":", const char *postLine = "" ) :
            ErrorFormatter( new Adapter(o), preLine, postLine ) {}
        virtual ~StdioPrinter() { delete outputStream(); }

    private:
        class Adapter : public OutputStream
        {
            FILE *_o;
        public:
            Adapter( FILE *o ) : _o(o) {}
            virtual void flush() { fflush( _o ); }
            virtual OutputStream &operator<<( unsigned i ) { fprintf( _o, "%u", i ); return *this; }
            virtual OutputStream &operator<<( const char *s ) { fputs( s, _o ); return *this; }
        };
    };
}

#endif // __CXXTEST__STDIOPRINTER_H
