#ifndef __CXXTEST__TEELISTENER_H
#define __CXXTEST__TEELISTENER_H

//
// A TeeListener notifies two "reular" TestListeners
//

#include <cxxtest/TestListener.h>

namespace CxxTest
{
    class TeeListener : public TestListener
    {
    public:
	TeeListener()
	{
	    setFirst( 0 );
	    setSecond( 0 );
	}

	virtual ~TeeListener()
	{
	}

	void setFirst( TestListener *first = 0 )
	{
	    _first = fixPointer( first );
	}

	void setSecond( TestListener *second = 0 )
	{
	    _second = fixPointer( second );
	}

    	void enterWorld( const WorldDescription &d )
	{
	    _first->enterWorld( d );
	    _second->enterWorld( d );
	}

	void enterSuite( const SuiteDescription &d )
	{
	    _first->enterSuite( d );
	    _second->enterSuite( d );
	}
	
	void enterTest( const TestDescription &d )
	{
	    _first->enterTest( d );
	    _second->enterTest( d );
	}
	
        void warning( const char *file, unsigned line, const char *expression )
	{
	    _first->warning( file, line, expression );
	    _second->warning( file, line, expression );
	}
	
        void failedTest( const char *file, unsigned line, const char *expression )
	{
	    _first->failedTest( file, line, expression );
	    _second->failedTest( file, line, expression );
	}
	
	void failedAssert( const char *file, unsigned line, const char *expression )
	{
	    _first->failedAssert( file, line, expression );
	    _second->failedAssert( file, line, expression );
	}
	
	void failedAssertEquals( const char *file, unsigned line,
				 const char *xStr, const char *yStr,
				 const char *x, const char *y )
	{
	    _first->failedAssertEquals( file, line, xStr, yStr, x, y );
	    _second->failedAssertEquals( file, line, xStr, yStr, x, y );
	}
	
	void failedAssertDelta( const char *file, unsigned line,
				const char *xStr, const char *yStr, const char *dStr,
				const char *x, const char *y, const char *d )
	{
	    _first->failedAssertDelta( file, line, xStr, yStr, dStr, x, y, d );
	    _second->failedAssertDelta( file, line, xStr, yStr, dStr, x, y, d );
	}
	
	void failedAssertDiffers( const char *file, unsigned line,
				  const char *xStr, const char *yStr,
				  const char *value )
	{
	    _first->failedAssertDiffers( file, line, xStr, yStr, value );
	    _second->failedAssertDiffers( file, line, xStr, yStr, value );
	}
	
	void failedAssertLessThan( const char *file, unsigned line,
				   const char *xStr, const char *yStr,
				   const char *x, const char *y )
	{
	    _first->failedAssertLessThan( file, line, xStr, yStr, x, y );
	    _second->failedAssertLessThan( file, line, xStr, yStr, x, y );
	}
	
	void failedAssertThrows( const char *file, unsigned line,
				 const char *expression, const char *type,
				 bool otherThrown )
	{
	    _first->failedAssertThrows( file, line, expression, type, otherThrown );
	    _second->failedAssertThrows( file, line, expression, type, otherThrown );
	}
	
	void failedAssertThrowsNot( const char *file, unsigned line,
				    const char *expression )
	{
	    _first->failedAssertThrowsNot( file, line, expression );
	    _second->failedAssertThrowsNot( file, line, expression );
	}
	
	void leaveTest( const TestDescription &d )
	{
	    _first->leaveTest(d);
	    _second->leaveTest(d);
	}
	
	void leaveSuite( const SuiteDescription &d )
	{
	    _first->leaveSuite(d);
	    _second->leaveSuite(d);
	}
	
	void leaveWorld( const WorldDescription &d )
	{
	    _first->leaveWorld(d);
	    _second->leaveWorld(d);
	}

    private:
	TestListener *_first, *_second;
    };
};


#endif // __CXXTEST__TEELISTENER_H
