#ifndef __CXXTEST__TESTLISTENER_H
#define __CXXTEST__TESTLISTENER_H

//
// TestListener is the base class for all "listeners",
// i.e. classes that receive notifications of the
// testing process.
//

#include <cxxtest/Descriptions.h>
#include <cxxtest/TestTracker.h>

namespace CxxTest
{
    class TestListener
    {
	static TestListener _dummy;
        static TestListener *_listener;
	
    public:
	static TestListener *fixPointer( TestListener *l ) { return l ? l : &_dummy; }
        static void setListener( TestListener *l ) { _listener = fixPointer(l); }
	static TestListener &listener() { return *_listener; }

	virtual void enterWorld( const WorldDescription &desc ) {}
	virtual void enterSuite( const SuiteDescription &desc ) {}
	virtual void enterTest( const TestDescription &desc ) {}
        virtual void warning( const char *file, unsigned line,
                              const char *expression ) {}
        virtual void failedTest( const char *file, unsigned line,
                                 const char *expression ) {}
	virtual void failedAssert( const char *file, unsigned line,
				   const char *expression ) {}
	virtual void failedAssertEquals( const char *file, unsigned line,
					 const char *xStr, const char *yStr,
					 const char *x, const char *y ) {}
	virtual void failedAssertDelta( const char *file, unsigned line,
                                        const char *xStr, const char *yStr, const char *dStr,
                                        const char *x, const char *y, const char *d ) {}
	virtual void failedAssertDiffers( const char *file, unsigned line,
                                          const char *xStr, const char *yStr,
                                          const char *value ) {}
	virtual void failedAssertLessThan( const char *file, unsigned line,
					   const char *xStr, const char *yStr,
					   const char *x, const char *y ) {}
	virtual void failedAssertThrows( const char *file, unsigned line,
                                         const char *expression, const char *type,
                                         bool otherThrown ) {}
	virtual void failedAssertThrowsNot( const char *file, unsigned line,
					    const char *expression ) {}
	virtual void leaveTest( const TestDescription &desc ) {}
	virtual void leaveSuite( const SuiteDescription &desc ) {}
	virtual void leaveWorld( const WorldDescription &desc ) {}
    };
}

#endif // __CXXTEST__TESTLISTENER_H
