#ifndef __TESTTRACKER_H
#define __TESTTRACKER_H

//
// The TestTracker tracks running tests
// The actual work is done in CountingListenerProxy,
// but this way avoids cyclic references TestListener<->CountingListenerProxy
//

#include <cxxtest/Descriptions.h>

namespace CxxTest
{
    class TestListener;
    
    class TestTracker
    {
        static TestTracker _dummy;
        static TestTracker *_tracker;
    public:
        static TestTracker *fixPointer( TestTracker *t ) { return t ? t : &_dummy; }
        static void setTracker( TestTracker *t ) { _tracker = fixPointer(t); }
	static TestTracker &tracker() { return *_tracker; }

	virtual const TestDescription &test() const { return TestDescription::_dummy; }
	virtual const SuiteDescription &suite() const { return SuiteDescription::_dummy; }
	virtual const WorldDescription &world() const { return WorldDescription::_dummy; }
        virtual bool testFailed() const { return (testFailedAsserts() > 0); }
        virtual bool suiteFailed() const { return (suiteFailedTests() > 0); }
        virtual bool worldFailed() const { return (failedSuites() > 0); }
        virtual unsigned testFailedAsserts() const { return 0; }
        virtual unsigned suiteFailedTests() const { return 0; }
        virtual unsigned failedSuites() const { return 0; }
        virtual unsigned failedTests() const { return 0; }
    };
};


#endif // __TESTTRACKER_H
